@@ -9,47 +9,18 @@ use strict;
 use POSIX qw(ENOENT EACCES EBADF);
 use Fcntl qw(SEEK_CUR SEEK_SET SEEK_END O_RDWR O_CREAT O_TRUNC);
 
-# Try and use IO::AIO or Linux::AIO, if it's around.
+# Try and use IO::AIO, if it's around.
 BEGIN {
     $Perlbal::OPTMOD_IO_AIO        = eval "use IO::AIO 1.6 (); 1;";
-    $Perlbal::OPTMOD_LINUX_AIO     = eval "use Linux::AIO 1.71 (); 1;";
-
-    # let's test if their Linux::AIO works on their platform.  common for it not to.
-    if ($Perlbal::OPTMOD_LINUX_AIO) {
-        my $good = 0;
-        Linux::AIO::aio_open("/tmp/$$-" . rand() . "-bogusdir/bogusfile-$$", O_RDWR|O_CREAT|O_TRUNC, 0, sub {
-            $good = 1 if $_[0] < 0 && $! == ENOENT;
-        });
-        while (Linux::AIO::nreqs()) {
-            my $rfd = "";
-            vec ($rfd, Linux::AIO::poll_fileno(), 1) = 1;
-            select $rfd, undef, undef, undef;
-            Linux::AIO::poll_cb();
-        }
-        unless ($good) {
-            # pretend that they don't have Linux::AIO, but only bitch at them if they don't have IO::AIO ...
-            unless ($Perlbal::OPTMOD_IO_AIO) {
-                warn("WARNING:  Your installation of Linux::AIO doesn't work.\n".
-                     "          You seem to have installed it without 'make test',\n".
-                     "          or you ignored the failing tests.  I'm going to ignore\n".
-                     "          that you have it and proceed without async IO.  The\n".
-                     "          modern replacement to Linux::AIO is IO::AIO.\n");
-            }
-            $Perlbal::OPTMOD_LINUX_AIO = 0;
-        }
-    }
 }
 
 END {
-    Linux::AIO::max_parallel(0)
-        if $Perlbal::OPTMOD_LINUX_AIO;
     IO::AIO::max_parallel(0)
         if $Perlbal::OPTMOD_IO_AIO;
 }
 
 $Perlbal::AIO_MODE = "none";
 $Perlbal::AIO_MODE = "ioaio" if $Perlbal::OPTMOD_IO_AIO;
-$Perlbal::AIO_MODE = "linux" if $Perlbal::OPTMOD_LINUX_AIO;
 
 ############################################################################
 # AIO functions available to callers
@@ -60,9 +31,7 @@ sub aio_readahead {
 
     aio_channel_push(get_chan(), $user_cb, sub {
         my $cb = shift;
-        if ($Perlbal::AIO_MODE eq "linux") {
-            Linux::AIO::aio_readahead($fh, $offset, $length, $cb);
-        } elsif ($Perlbal::AIO_MODE eq "ioaio") {
+        if ($Perlbal::AIO_MODE eq "ioaio") {
             IO::AIO::aio_readahead($fh, $offset, $length, $cb);
         } else {
             $cb->();
@@ -75,9 +44,7 @@ sub aio_stat {
 
     aio_channel_push(get_chan($file), $user_cb, sub {
         my $cb = shift;
-        if ($Perlbal::AIO_MODE eq "linux") {
-            Linux::AIO::aio_stat($file, $cb);
-        } elsif ($Perlbal::AIO_MODE eq "ioaio") {
+        if ($Perlbal::AIO_MODE eq "ioaio") {
             IO::AIO::aio_stat($file, $cb);
         } else {
             stat($file);
@@ -92,13 +59,7 @@ sub aio_open {
     aio_channel_push(get_chan($file), $user_cb, sub {
         my $cb = shift;
 
-        if ($Perlbal::AIO_MODE eq "linux") {
-            Linux::AIO::aio_open($file, $flags, $mode, sub {
-                my $fd = shift;
-                my $fh = _fh_of_fd_mode($fd, $mode);
-                $cb->($fh);
-            });
-        } elsif ($Perlbal::AIO_MODE eq "ioaio") {
+        if ($Perlbal::AIO_MODE eq "ioaio") {
             IO::AIO::aio_open($file, $flags, $mode, $cb);
         } else {
             my $fh;
@@ -113,9 +74,7 @@ sub aio_unlink {
     aio_channel_push(get_chan($file), $user_cb, sub {
         my $cb = shift;
 
-        if ($Perlbal::AIO_MODE eq "linux") {
-            Linux::AIO::aio_unlink($file, $cb);
-        } elsif ($Perlbal::AIO_MODE eq "ioaio") {
+        if ($Perlbal::AIO_MODE eq "ioaio") {
             IO::AIO::aio_unlink($file, $cb);
         } else {
             my $rv = unlink($file);
@@ -133,9 +92,7 @@ sub aio_write {
 
     aio_channel_push(get_chan(), $user_cb, sub {
         my $cb = shift;
-        if ($Perlbal::AIO_MODE eq "linux") {
-            Linux::AIO::aio_write($fh, $offset, $length, $alist->[3], 0, $cb);
-        } elsif ($Perlbal::AIO_MODE eq "ioaio") {
+        if ($Perlbal::AIO_MODE eq "ioaio") {
             IO::AIO::aio_write($fh, $offset, $length, $alist->[3], 0, $cb);
         } else {
             my $old_off = sysseek($fh, 0, SEEK_CUR);
@@ -155,9 +112,7 @@ sub aio_read {
 
     aio_channel_push(get_chan(), $user_cb, sub {
         my $cb = shift;
-        if ($Perlbal::AIO_MODE eq "linux") {
-            Linux::AIO::aio_read($fh, $offset, $length, $alist->[3], 0, $cb);
-        } elsif ($Perlbal::AIO_MODE eq "ioaio") {
+        if ($Perlbal::AIO_MODE eq "ioaio") {
             IO::AIO::aio_read($fh, $offset, $length, $alist->[3], 0, $cb);
         } else {
             my $old_off = sysseek($fh, 0, SEEK_CUR);
